@@ -49,10 +49,8 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 
   /* Search the instance and update its name */
 
-  if(myGlobals.device[actualDeviceId].virtualDevice) return;
-
   for(el=getFirstHost(actualDeviceId); el != NULL; el = getNextHost(actualDeviceId, el)) {
-    if((el->hostNumIpAddress != NULL) && (addrcmp(&el->hostIpAddress, &addr) == 0)) {
+    if(addrcmp(&el->hostIpAddress, &addr) == 0) {
       accessAddrResMutex("updateHostNameInfo");
 
       if(el != NULL) {
@@ -64,6 +62,8 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 	for(i=0; i<strlen(symbolic); i++)
 	  if(isupper(symbolic[i])) symbolic[i] = tolower(symbolic[i]);
 
+	/* traceEvent(CONST_TRACE_INFO, "--> %s", symbolic); */
+
 	setResolvedName(el, symbolic, type);
       }
 
@@ -74,11 +74,13 @@ static void updateDeviceHostNameInfo(HostAddr addr, char* symbolic, int actualDe
 
 /* **************************************** */
 
-  static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
+static void updateHostNameInfo(HostAddr addr, char* symbolic, int type) {
   int i;
 
-  for(i=0; i<myGlobals.numDevices; i++)
-    updateDeviceHostNameInfo(addr, symbolic, i, type);
+  for(i=0; i<myGlobals.numDevices; i++) {
+    if(!myGlobals.device[i].virtualDevice)
+      updateDeviceHostNameInfo(addr, symbolic, i, type);
+  }
 }
 
 /* ************************************ */
@@ -670,7 +672,7 @@ void* dequeueAddress(void *_i) {
         break;
 
       if (size == sizeof(struct in_addr)) {
-	/*addrput(AF_INET, &addr, data_data.dptr);*/
+	/* addrput(AF_INET, &addr, data_data.dptr); */
 	addr.hostFamily = AF_INET;
 	memcpy(&addr.Ip4Address.s_addr, data_data.dptr, size);
       }
@@ -707,7 +709,13 @@ void* dequeueAddress(void *_i) {
   myGlobals.dequeueAddressThreadId[dqaIndex] = 0;
 
   traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: DNSAR(%d): Address resolution thread terminated [p%d]",
-             pthread_self(), dqaIndex+1, getpid());
+             pthread_self(), dqaIndex+1, 
+#ifndef WIN32
+			 getpid()
+#else
+			 0
+#endif
+			 );
 
   return(NULL);
 }
@@ -908,7 +916,7 @@ int fetchAddressFromCache(HostAddr hostIpAddress, char *buffer, int *type) {
 /* This function automatically updates the instance name */
 
 void ipaddr2str(HostAddr hostIpAddress, int updateHost) {
-  char buf[MAX_LEN_SYM_HOST_NAME+1];
+  char buf[MAX_LEN_SYM_HOST_NAME+1] = { '\0' };
   int type;
 
   myGlobals.numipaddr2strCalls++;
@@ -1623,7 +1631,7 @@ u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
        * E.g. : 89.10.67.213.in-addr.arpa
        *
        */
-      char *a, *b, *c, *d, dnsBuf[128], *strtokState;
+      char *a, *b, *c, *d, dnsBuf[128], *strtokState = NULL;
       unsigned long theDNSaddr;
 
       len = strlen((char*)bp);
@@ -1834,15 +1842,92 @@ void checkSpoofing(HostTraffic *hostToCheck, int actualDeviceId) {
 
 /* **************************************** */
 
-char* host2networkName(HostTraffic *el, char *buf, u_short buf_len) {
-  struct in_addr addr = el->hostIpAddress.Ip4Address;
+char* subnetId2networkName(int8_t known_subnet_id, char *buf, u_short buf_len) {
+  struct in_addr addr;
   char buf1[64];
       
-  addr.s_addr = addr.s_addr & (0xFFFFFFFF << (32-el->network_mask));
+  if((known_subnet_id == UNKNOWN_SUBNET_ID)
+     || (known_subnet_id < 0)
+     || (known_subnet_id >= myGlobals.numKnownSubnets))
+    safe_snprintf(__FILE__, __LINE__, buf, buf_len, "0.0.0.0/0");
+  else {
+    addr.s_addr = myGlobals.subnetStats[known_subnet_id].address[CONST_NETWORK_ENTRY];
+    
+    safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s/%d",
+		  _intoa(addr, buf1, sizeof(buf1)), 
+		  myGlobals.subnetStats[known_subnet_id].address[CONST_NETMASK_V6_ENTRY]);
+  }
 
-  safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s/%d",
-		_intoa(addr, buf1, sizeof(buf1)), 
-		el->network_mask);
+  return(buf);
+}
+
+/* **************************************** */
+
+char* host2networkName(HostTraffic *el, char *buf, u_short buf_len) {
+  buf[0] = '\0';
+
+  if(el != NULL) {
+    if(el->known_subnet_id != UNKNOWN_SUBNET_ID)
+      return(subnetId2networkName(el->known_subnet_id, buf, buf_len));
+    else if((el->network_mask > 0) && (el->hostIpAddress.hostFamily == AF_INET)) {
+      struct in_addr addr;
+      char buf1[32];
+
+      addr.s_addr = el->hostIpAddress.Ip4Address.s_addr & (~(0xFFFFFFFF >> el->network_mask));
+
+      safe_snprintf(__FILE__, __LINE__, buf, buf_len, "%s/%d",
+		    _intoa(addr, buf1, sizeof(buf1)),
+		    el->network_mask);
+    }
+  }
   
   return(buf);
 }
+
+/* **************************************** */
+
+void addDeviceNetworkToKnownSubnetList(NtopInterface *device) {
+  int i;
+
+  if(device->network.s_addr == 0) return;
+
+  for(i=0; i<myGlobals.numKnownSubnets; i++) {
+    if((device->network.s_addr == myGlobals.subnetStats[i].address[CONST_NETWORK_ENTRY])
+       && (device->netmask.s_addr == myGlobals.subnetStats[i].address[CONST_NETMASK_ENTRY]))
+       return; /* Already present */
+  }
+
+  /* Not present: we add it to the list */
+  if(myGlobals.numKnownSubnets >= (MAX_NUM_NETWORKS-1)) {
+    traceEvent(CONST_TRACE_WARNING, "Too many known subnets defined (%d)",
+	       myGlobals.numKnownSubnets);
+    return;
+  } else {
+    i = myGlobals.numKnownSubnets;
+    myGlobals.subnetStats[i].address[CONST_NETWORK_ENTRY]    = device->network.s_addr;
+    myGlobals.subnetStats[i].address[CONST_NETMASK_ENTRY]    = device->netmask.s_addr;
+    myGlobals.subnetStats[i].address[CONST_NETMASK_V6_ENTRY] = num_network_bits(device->netmask.s_addr);
+    myGlobals.subnetStats[i].address[CONST_BROADCAST_ENTRY]  = device->network.s_addr | (~device->netmask.s_addr);
+    myGlobals.numKnownSubnets++;
+  }
+}
+
+/* **************************************** */
+
+void updateHostKnownSubnet(HostTraffic *el) {
+  int i;
+
+  if(el->hostIpAddress.hostFamily != AF_INET /* v4 */) return;
+
+  for(i=0; i<myGlobals.numKnownSubnets; i++) {
+    if((el->hostIpAddress.addr._hostIp4Address.s_addr & myGlobals.subnetStats[i].address[CONST_NETMASK_ENTRY])
+       == myGlobals.subnetStats[i].address[CONST_NETWORK_ENTRY]) {
+      el->known_subnet_id = i;
+      // FD_SET(FLAG_SUBNET_LOCALHOST, &el->flags);
+      FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
+      return;
+    }
+  }
+
+  el->known_subnet_id = UNKNOWN_SUBNET_ID;
+}
