@@ -28,7 +28,7 @@
 
   Many thanks Philippe!
 */
-char *languages[] = { "", "perl", "php", "xml", "python", "no" };
+char *languages[] = { "", "perl", "php", "xml", "python", "json", "no" };
 
 /* *************************** */
 
@@ -58,6 +58,9 @@ static void initWriteArray(FILE *fDescr, int lang) {
   case FLAG_PYTHON_LANGUAGE:
     sendEmitterString(fDescr, "ntopDict = {\n");
     break ;
+  case FLAG_JSON_LANGUAGE:
+    sendEmitterString(fDescr, "{ \"ntop\": [");
+    break;
   case FLAG_XML_LANGUAGE:
     sendEmitterString(fDescr, "<rpc-reply xmlns:ntop=\"http://www.ntop.org/ntop.dtd\">"
 		      "\n<ntop-traffic-information>\n");
@@ -69,7 +72,9 @@ static void initWriteArray(FILE *fDescr, int lang) {
 
 /* *************************** */
 
-static void endWriteArray(FILE *fDescr, int lang) {
+static void endWriteArray(FILE *fDescr, int lang, unsigned int numEntries) {
+  char buf[256];
+
   switch(lang) {
   case FLAG_PERL_LANGUAGE:
   case FLAG_PHP_LANGUAGE:
@@ -78,6 +83,11 @@ static void endWriteArray(FILE *fDescr, int lang) {
   case FLAG_PYTHON_LANGUAGE:
     sendEmitterString(fDescr, "}\n");
     break;
+  case FLAG_JSON_LANGUAGE:
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), 
+		  "], \"totalRecords\":%d\n}\n", numEntries);
+    sendEmitterString(fDescr, buf);
+    break;
   case FLAG_XML_LANGUAGE:
     sendEmitterString(fDescr, "</ntop-traffic-information>\n</rpc-reply>\n");
     break ;
@@ -124,6 +134,10 @@ static void initWriteKey(FILE *fDescr, int lang, char *indent,
     safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s': {\n",indent, keyName);
     sendEmitterString(fDescr, buf);
     break ;
+  case FLAG_JSON_LANGUAGE:
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "{ \"host_key\": \"%s\",", keyName);
+    sendEmitterString(fDescr, buf);
+    break;
   case FLAG_XML_LANGUAGE:
     safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s<%s>\n", indent, keyName);
     sendEmitterString(fDescr, buf);
@@ -163,9 +177,18 @@ static void endWriteKey(FILE *fDescr, int lang, char *indent, char *keyName, cha
     sendEmitterString(fDescr, buf);
     break ;
   case FLAG_PYTHON_LANGUAGE:
-    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"%s}%c\n",indent,last);
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"%s}%c\n",indent, last);
     sendEmitterString(fDescr, buf);
     break ;
+  case FLAG_JSON_LANGUAGE:
+    if(last) {
+      safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf)," \"dummy\": 1");
+      sendEmitterString(fDescr, buf);
+    }
+
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),"}%c\n", last);
+    sendEmitterString(fDescr, buf);
+    break;
   case FLAG_NO_LANGUAGE:
     if(strcmp(indent, "") == 0) sendEmitterString(fDescr, "\n");
     break ;
@@ -194,41 +217,34 @@ static char* sanitize(char *value, char *buf, int buf_len) {
 
 /* *************************** */
 
-static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
+static void wrtKV(FILE *fDescr, int lang, char *indent, char *name,
 		      char *value, char last, int numEntriesSent) {
-  char buf[256], buf1[256];
+  char buf[256];
 
   validateString(name);
 
   switch(lang) {
   case FLAG_PERL_LANGUAGE:
   case FLAG_PHP_LANGUAGE:
-    /* In the case of hostNumIpAddress and hostResolvedName,
-       the pointer is not null, but the string is empty.
-       In that case, don't create the key in the array.
-    */
-    if((value != NULL) && (value[0] != '\0')) {
-	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s' => '%s'%c\n",
-		      indent, name, sanitize(value, buf1, sizeof(buf1)), last);
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s' => %s%c\n",
+		      indent, name, value, last);
 	sendEmitterString(fDescr, buf);
-      }
     break ;
   case FLAG_XML_LANGUAGE:
-    if((value != NULL) && (value[0] != '\0'))
-      {
 	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s<%s>%s</%s>\n", 
-		      indent, name, sanitize(value, buf1, sizeof(buf1)), name);
+		       indent, name, value, name);
 	sendEmitterString(fDescr, buf);
-      }
     break ;
   case FLAG_PYTHON_LANGUAGE:
-    if((value != NULL) && (value[0] != '\0'))
-      {
-	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s': '%s'%c\n", 
-		      indent, name, sanitize(value, buf1, sizeof(buf1)), last);
+	safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s'%s': %s%c\n", 
+		      indent, name, value, last);
  	sendEmitterString(fDescr, buf);
-       }
      break ;
+  case FLAG_JSON_LANGUAGE:
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), " \"%s\": %s,", 
+		  name, value);
+    sendEmitterString(fDescr, buf);
+     break ;    
   case FLAG_NO_LANGUAGE:
     if(value != NULL) {
       safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s|", 
@@ -245,21 +261,36 @@ static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
 
 /* *************************** */
 
-static void wrtIntItm(FILE *fDescr, int lang, char *indent, char *name,
-		      int value, char last, int numEntriesSent) {
-  char buf[80];
-  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
+static void wrtStrItm(FILE *fDescr, int lang, char *indent, char *name,
+              char *value, char last, int numEntriesSent) {
+  char buf[256], buf1[256];
+  if ((value != NULL) && (value[0] != '\0')) {
+    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), 
+		  (lang == FLAG_XML_LANGUAGE) ? "%s" : ((lang == FLAG_JSON_LANGUAGE) ? "\"%s\"" : "'%s'"),
+                sanitize(value, buf1, sizeof(buf1)));
+    wrtKV(fDescr, lang, indent, name, buf, last, numEntriesSent);
+  }
 }
 
 /* *************************** */
 
+static void wrtIntItm(FILE *fDescr, int lang, char *indent, char *name,
+		      int value, char last, int numEntriesSent) {
+  char buf[80];
+  safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
+  wrtKV(fDescr, lang, indent, name, buf, last, numEntriesSent);
+}
+
+/* *************************** */
+
+#if 0
 static void wrtIntStrItm(FILE *fDescr, int lang, char *indent,int name,
 			 char *value, char useless, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",name);
-  wrtStrItm(fDescr, lang, indent, buf, value, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, buf, value, ',', numEntriesSent);
 }
+#endif
 
 /* *************************** */
 
@@ -267,7 +298,7 @@ static void wrtUintItm(FILE *fDescr, int lang, char *indent, char *name,
 		       unsigned int value, char useless, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
 /* *************************** */
@@ -276,7 +307,7 @@ static void wrtUcharItm(FILE *fDescr, int lang, char *indent, char *name,
 			u_char value, char useless, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
 /* *************************** */
@@ -285,7 +316,7 @@ static void wrtFloatItm(FILE *fDescr, int lang, char *indent, char *name,
 			float value, char last, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%0.2f",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, last, numEntriesSent);
 }
 
 /* *************************** */
@@ -305,7 +336,7 @@ static void wrtUlongItm(FILE *fDescr, int lang, char *indent, char *name,
   char buf[80];
 
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%lu",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
 /* *************************** */
@@ -315,7 +346,7 @@ static void wrtLlongItm(FILE *fDescr, int lang, char* indent, char* name,
   char buf[80];
 
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf),  "%llu", value.value);
-  wrtStrItm(fDescr, lang, indent, name, buf, last, numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, last, numEntriesSent);
 }
 
 /* *************************** */
@@ -324,7 +355,7 @@ static void wrtTime_tItm(FILE *fDescr, int lang, char *indent, char *name,
 			 time_t value, char useless, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%ld",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
 /* *************************** */
@@ -333,7 +364,7 @@ static void wrtUshortItm(FILE *fDescr, int lang, char *indent, char *name,
 			 u_short value, char useless, int numEntriesSent) {
   char buf[80];
   safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%d",value);
-  wrtStrItm(fDescr, lang, indent, name, buf, ',', numEntriesSent);
+  wrtKV(fDescr, lang, indent, name, buf, ',', numEntriesSent);
 }
 
 /* ********************************** */
@@ -408,7 +439,7 @@ void dumpNtopFlows(FILE *fDescr, char* options, int actualDeviceId) {
     }
   }
 
-  if(numEntries > 0) endWriteArray(fDescr, lang);
+  if(numEntries > 0) endWriteArray(fDescr, lang, numEntries);
 }
 
 /* ********************************** */
@@ -488,9 +519,7 @@ void dumpNtopTrafficMatrix(FILE *fDescr, char* options, int actualDeviceId) {
 	}
       }
 
-
-
-  if(numEntries > 0) endWriteArray(fDescr, lang);
+  if(numEntries > 0) endWriteArray(fDescr, lang, numEntries);
 }
 
 /* ********************************** */
@@ -649,10 +678,10 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
     if(checkFilter(filter, "pktRcvd"))
       wrtLlongItm(fDescr, lang, "\t", "pktRcvd", el->pktRcvd, ',', numEntries);
 
-    if(checkFilter(filter, "ipBytesSent"))
-      wrtLlongItm(fDescr, lang, "\t", "ipBytesSent", el->ipBytesSent, ',', numEntries);
-    if(checkFilter(filter, "ipBytesRcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "ipBytesRcvd", el->ipBytesRcvd, ',', numEntries);
+    if(checkFilter(filter, "ipv4BytesSent"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesSent", el->ipv4BytesSent, ',', numEntries);
+    if(checkFilter(filter, "ipv4BytesRcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesRcvd", el->ipv4BytesRcvd, ',', numEntries);
 
     /* *************************************** */
 
@@ -745,18 +774,17 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 	wrtFloatItm(fDescr, lang, "\t", "averageTPktThpt",el->averageTPktThpt, ',', numEntries);
       if(checkFilter(filter, "peakTPktThpt"))
 	wrtFloatItm(fDescr, lang, "\t", "peakTPktThpt", el->peakTPktThpt, ',', numEntries);
-
     }
 
-    if(checkFilter(filter, "ipBytesSent"))
-      wrtLlongItm(fDescr, lang, "\t", "ipBytesSent", el->ipBytesSent, ',', numEntries);
-    if(checkFilter(filter, "ipBytesRcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "ipBytesRcvd", el->ipBytesRcvd, ',', numEntries);
+    if(checkFilter(filter, "ipv4BytesSent"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesSent", el->ipv4BytesSent, ',', numEntries);
+    if(checkFilter(filter, "ipv4BytesRcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv4BytesRcvd", el->ipv4BytesRcvd, ',', numEntries);
 
-    if(checkFilter(filter, "ipv6Sent"))
-      wrtLlongItm(fDescr, lang, "\t", "ipv6Sent", el->ipv6Sent, ',', numEntries);
-    if(checkFilter(filter, "ipv6Rcvd"))
-      wrtLlongItm(fDescr, lang, "\t", "ipv6Rcvd", el->ipv6Rcvd, ',', numEntries);
+    if(checkFilter(filter, "ipv6BytesSent"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv6BytesSent", el->ipv6BytesSent, ',', numEntries);
+    if(checkFilter(filter, "ipv6BytesRcvd"))
+      wrtLlongItm(fDescr, lang, "\t", "ipv6BytesRcvd", el->ipv6BytesRcvd, ',', numEntries);
 
     ctr.value = el->tcpSentLoc.value+el->tcpSentRem.value;
     if(checkFilter(filter, "tcpBytesSent"))
@@ -809,7 +837,15 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 	wrtLlongItm(fDescr, lang, "\t", "icmpFragmentsSent", el->icmpFragmentsSent, ',', numEntries);
       if(checkFilter(filter, "icmpFragmentsRcvd"))
 	wrtLlongItm(fDescr, lang, "\t", "icmpFragmentsRcvd", el->icmpFragmentsRcvd, ',', numEntries);
-
+      if(checkFilter(filter, "greSent"))
+	wrtLlongItm(fDescr, lang, "\t", "greSent", el->greSent, ',', numEntries);
+      if(checkFilter(filter, "greRcvd"))
+	wrtLlongItm(fDescr, lang, "\t", "greRcvd", el->greRcvd, ',', numEntries);
+      if(checkFilter(filter, "ipsecSent"))
+	wrtLlongItm(fDescr, lang, "\t", "ipsecSent", el->ipsecSent, ',', numEntries);
+      if(checkFilter(filter, "ipsecRcvd"))
+	wrtLlongItm(fDescr, lang, "\t", "ipsecRcvd", el->ipsecRcvd, ',', numEntries);
+      
       /* ***************************** */
 
       if(el->nonIPTraffic != NULL) {
@@ -821,10 +857,6 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 	  wrtLlongItm(fDescr, lang, "\t", "ipxSent", el->nonIPTraffic->ipxSent, ',', numEntries);
 	if(checkFilter(filter, "ipxRcvd"))
 	  wrtLlongItm(fDescr, lang, "\t", "ipxRcvd", el->nonIPTraffic->ipxRcvd, ',', numEntries);
-	if(checkFilter(filter, "osiSent"))
-	  wrtLlongItm(fDescr, lang, "\t", "osiSent", el->nonIPTraffic->osiSent, ',', numEntries);
-	if(checkFilter(filter, "osiRcvd"))
-	  wrtLlongItm(fDescr, lang, "\t", "osiRcvd", el->nonIPTraffic->osiRcvd, ',', numEntries);
 	if(checkFilter(filter, "dlcSent"))
 	  wrtLlongItm(fDescr, lang, "\t", "dlcSent", el->nonIPTraffic->dlcSent, ',', numEntries);
 	if(checkFilter(filter, "dlcRcvd"))
@@ -840,10 +872,6 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 	  wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsSent", el->nonIPTraffic->arpReplyPktsSent, ',', numEntries);
 	if(checkFilter(filter, "arpReplyPktsRcvd"))
 	  wrtLlongItm(fDescr, lang, "\t", "arpReplyPktsRcvd", el->nonIPTraffic->arpReplyPktsRcvd, ',', numEntries);
-	if(checkFilter(filter, "decnetSent"))
-	  wrtLlongItm(fDescr, lang, "\t", "decnetSent", el->nonIPTraffic->decnetSent, ',', numEntries);
-	if(checkFilter(filter, "decnetRcvd"))
-	  wrtLlongItm(fDescr, lang, "\t", "decnetRcvd", el->nonIPTraffic->decnetRcvd, ',', numEntries);
 	if(checkFilter(filter, "appletalkSent"))
 	  wrtLlongItm(fDescr, lang, "\t", "appletalkSent", el->nonIPTraffic->appletalkSent, ',', numEntries);
 	if(checkFilter(filter, "appletalkRcvd"))
@@ -1091,7 +1119,7 @@ void dumpNtopHashes(FILE *fDescr, char* options, int actualDeviceId) {
 
   if(numEntries > 0) endWriteKey(fDescr, lang,"", (lang == FLAG_XML_LANGUAGE) ? "host-information" : hostKey, ' ');
 
-  endWriteArray(fDescr, lang);
+  endWriteArray(fDescr, lang, numEntries);
 }
 
 /* ********************************** */
@@ -1130,6 +1158,9 @@ void dumpNtopHashIndexes(FILE *fDescr, char* options, int actualDeviceId) {
   }
 
   initWriteArray(fDescr, lang);
+  
+  if (lang == FLAG_XML_LANGUAGE)
+    sendEmitterString(fDescr, "<keys>\n");
 
   for(el=getFirstHost(actualDeviceId);
       el != NULL; el = getNextHost(actualDeviceId, el)) {
@@ -1138,21 +1169,31 @@ void dumpNtopHashIndexes(FILE *fDescr, char* options, int actualDeviceId) {
 
     if(!broadcastHost(el)) {
       char *hostKey;
-
-      if(el->hostNumIpAddress[0] != '\0')
-	hostKey = el->hostNumIpAddress;
+      char *hostName = "Unknown";
+      
+      if(el->hostNumIpAddress[0] != '\0') {
+        hostKey = el->hostNumIpAddress;
+        if ((el->hostResolvedName != NULL) && (el->hostResolvedName[0] != '\0'))
+          hostName = el->hostResolvedName;
+      }
+      else {
+        hostKey = el->ethAddressString;
+      }
+      if (lang == FLAG_XML_LANGUAGE)
+        wrtStrItm(fDescr, lang, "\t", "item", hostKey, '\n', numEntries);
       else
-	hostKey = el->ethAddressString;
-
-      wrtIntStrItm(fDescr, lang, "", 0, hostKey,'\n', numEntries);
+        wrtStrItm(fDescr, lang, "", hostKey, hostName, ',', numEntries);        
 
       numEntries++;
     }
 
     unlockHostsHashMutex(el);
   } /* for */
+  
+  if (lang == FLAG_XML_LANGUAGE)
+    sendEmitterString(fDescr, "</keys>\n");
 
-  endWriteArray(fDescr, lang);
+  endWriteArray(fDescr, lang, numEntries);
 }
 
 /* ********************************** */
@@ -1269,8 +1310,8 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       wrtLlongItm(fDescr, lang, "\t", "multicastPkts",myGlobals.device[i].multicastPkts, ',', numEntries);
     if(checkFilter(filter, "ethernetBytes"))
       wrtLlongItm(fDescr, lang, "\t", "ethernetBytes",myGlobals.device[i].ethernetBytes, ',', numEntries);
-    if(checkFilter(filter, "ipBytes"))
-      wrtLlongItm(fDescr, lang, "\t", "ipBytes",myGlobals.device[i].ipBytes, ',', numEntries);
+    if(checkFilter(filter, "ipv4Bytes"))
+      wrtLlongItm(fDescr, lang, "\t", "ipBytes",myGlobals.device[i].ipv4Bytes, ',', numEntries);
 
     if(!shortView) {
       if(checkFilter(filter, "fragmentedIpBytes"))
@@ -1292,8 +1333,8 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       wrtLlongItm(fDescr, lang, "\t", "ipxBytes",myGlobals.device[i].ipxBytes, ',', numEntries);
     if(checkFilter(filter, "stpBytes"))
       wrtLlongItm(fDescr, lang, "\t", "stpBytes",myGlobals.device[i].stpBytes, ',', numEntries);
-    if(checkFilter(filter, "decnetBytes"))
-      wrtLlongItm(fDescr, lang, "\t", "decnetBytes",myGlobals.device[i].decnetBytes, ',', numEntries);
+    if(checkFilter(filter, "ipsecBytes"))
+      wrtLlongItm(fDescr, lang, "\t", "ipsecBytes",myGlobals.device[i].ipsecBytes, ',', numEntries);
     if(checkFilter(filter, "netbiosBytes"))
       wrtLlongItm(fDescr, lang, "\t", "netbiosBytes",myGlobals.device[i].netbiosBytes, ',', numEntries);
     if(checkFilter(filter, "arpRarpBytes"))
@@ -1302,8 +1343,8 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
       wrtLlongItm(fDescr, lang, "\t", "atalkBytes",myGlobals.device[i].atalkBytes, ',', numEntries);
     if(checkFilter(filter, "egpBytes"))
       wrtLlongItm(fDescr, lang, "\t", "egpBytes",myGlobals.device[i].egpBytes, ',', numEntries);
-    if(checkFilter(filter, "osiBytes"))
-      wrtLlongItm(fDescr, lang, "\t", "osiBytes",myGlobals.device[i].osiBytes, ',', numEntries);
+    if(checkFilter(filter, "greBytes"))
+      wrtLlongItm(fDescr, lang, "\t", "greBytes",myGlobals.device[i].greBytes, ',', numEntries);
     if(checkFilter(filter, "ipv6Bytes"))
       wrtLlongItm(fDescr, lang, "\t", "ipv6Bytes",myGlobals.device[i].ipv6Bytes, ',', numEntries);
     if(checkFilter(filter, "otherBytes"))
@@ -1423,13 +1464,13 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
 	  if((myGlobals.ipTrafficProtosNames[j] != NULL) && (!shortView)) {
 	    if(j > 0) endWriteKey(fDescr, lang, "\t\t", hostKey, ',');
 	    initWriteKey(fDescr, lang, "\t\t", (hostKey = myGlobals.ipTrafficProtosNames[j]), numEntries);
-	    wrtLlongItm(fDescr, lang,"\t\t\t","local",
+	    wrtLlongItm(fDescr, lang, "\t\t\t", "local",
 			myGlobals.device[i].ipProtoStats[j].local, ',', numEntries);
-	    wrtLlongItm(fDescr, lang,"\t\t\t","local2remote",
+	    wrtLlongItm(fDescr, lang, "\t\t\t", "local2remote",
 			myGlobals.device[i].ipProtoStats[j].local2remote, ',', numEntries);
-	    wrtLlongItm(fDescr, lang,"\t\t\t","remote2local",
+	    wrtLlongItm(fDescr, lang, "\t\t\t", "remote2local",
 			myGlobals.device[i].ipProtoStats[j].remote2local, ',', numEntries);
-	    wrtLlongItm(fDescr, lang,"\t\t\t","remote",
+	    wrtLlongItm(fDescr, lang, "\t\t\t", "remote",
 			myGlobals.device[i].ipProtoStats[j].remote, ' ', numEntries);
 	  } else {
 	    TrafficCounter ctr;
@@ -1532,5 +1573,5 @@ void dumpNtopTrafficInfo(FILE *fDescr, char* options) {
   }
 
   if(numEntries > 0) endWriteKey(fDescr, lang, "", (lang == FLAG_XML_LANGUAGE) ? "device-information" : keyName, ' ');
-  endWriteArray(fDescr, lang);
+  endWriteArray(fDescr, lang, numEntries);
 }
