@@ -63,9 +63,6 @@ extern char *version, *osName, *author, *buildDate, *configureDate,
   *include_path,
   *system_libs,
   *install_path,
-#ifdef MAKE_WITH_I18N
-  *locale_dir,
-#endif
   *distro,
   *release,
   *force_runtime;
@@ -160,7 +157,10 @@ extern u_int16_t handleDNSpacket(HostTraffic *srcHost, u_short sport,
                                  short *isRequest, short *positiveReply);
 extern void checkSpoofing(HostTraffic *el, int actualDeviceId);
 extern void cleanupHostEntries(void);
+extern char* subnetId2networkName(int8_t known_subnet_id, char *buf, u_short buf_len);
 extern char* host2networkName(HostTraffic *el, char *buf, u_short buf_len);
+extern void updateHostKnownSubnet(HostTraffic *el);
+extern void addDeviceNetworkToKnownSubnetList(NtopInterface *device);
 
 /* admin.c */
 extern void showUsers(void);
@@ -251,23 +251,20 @@ extern void allocDeviceMemory(int deviceId);
 
 /* leaks.c */
 
-#define memorycheck(a, b) {}
-#define _memorycheck(a, b, c, d) {}
-
 #ifdef MAKE_WITH_SAFER_ROUTINES
 
 /* Fix to the free prototype courtesy of Tanner Lovelace <lovelace@opennms.org> */
 /* Fix free() courtesy of Benoit Dolez <bdolez@exosec.fr> */
-#define free(a)       { void *__t = (a); ntop_safefree((void**)&(__t), __FILE__, __LINE__); a = __t; }
+#define free(a)       { void *__t = (a); ntop_safefree((void**)&(__t), (char*)__FILE__, (int)__LINE__); a = __t; }
 extern void           ntop_safefree(void **ptr, char* file, int line);
-#define malloc(sz)    ntop_safemalloc(sz, __FILE__, __LINE__)
+#define malloc(sz)    ntop_safemalloc((unsigned int)sz, (char*)__FILE__, (int)__LINE__)
 extern void*          ntop_safemalloc(unsigned int sz, char* file, int line);
-#define calloc(c,sz)  ntop_safecalloc(c, sz, __FILE__, __LINE__)
+#define calloc(c,sz)  ntop_safecalloc(c, sz, (char*)__FILE__, (int)__LINE__)
 extern void*          ntop_safecalloc(unsigned int c, unsigned int sz, char* file, int line);
-#define realloc(p,sz) ntop_saferealloc(p, sz, __FILE__, __LINE__)
+#define realloc(p,sz) ntop_saferealloc(p, sz, (char*)__FILE__, (int)__LINE__)
 extern void*          ntop_saferealloc(void* ptr, unsigned int sz, char* file, int line);
 #undef strdup
-#define strdup(p)     ntop_safestrdup(p, __FILE__, __LINE__)
+#define strdup(p)     ntop_safestrdup(p, (char*)__FILE__, (int)__LINE__)
 extern char* ntop_safestrdup(char *ptr, char* file, int line);
 
 #elif defined(MEMORY_DEBUG) && (MEMORY_DEBUG == 1)
@@ -339,6 +336,14 @@ extern void           ntop_free(void **ptr, char* file, int line);
 #else
 #endif /* MAKE_WITH_SAFER_ROUTINES / MEMORY_DEBUG */
 
+#ifndef memorycheck
+#define memorycheck(a, b) {}
+#endif
+
+#ifndef _memorycheck
+#define _memorycheck(a, b, c, d) {}
+#endif
+
 /* Serialized replacements for gdbm routines... */
 
 #define gdbm_firstkey(a)             ntop_gdbm_firstkey(a, __FILE__, __LINE__)
@@ -376,8 +381,11 @@ extern RETSIGTYPE cleanup(int signo);
 /* pbuf.c */
 extern void allocateSecurityHostPkts(HostTraffic *srcHost);
 extern int handleIP(u_short port, HostTraffic *srcHost, HostTraffic *dstHost,
-		    const u_int _length, u_short isPassiveSess, u_short isVoipSess,
-		    u_short p2pSessionIdx, int actualDeviceId, u_short newSession);
+		    const u_int numPkts, const u_int _length,
+		    u_short isPassiveSess, u_short isVoipSess,
+		    u_short p2pSessionIdx, int actualDeviceId, u_short newSession,
+		    u_int efficiencySent /* 0 = unknown */,
+		    u_int efficiencyRcvd /* 0 = unknown */);
 extern void deleteFragment(IpFragment *fragment, int actualDeviceId);
 extern void purgeOldFragmentEntries(int actualDeviceId);
 extern void updateHostName(HostTraffic *el);
@@ -388,7 +396,11 @@ extern void updatePacketCount(HostTraffic *srcHost, HostAddr *srcAddr,
 			      HostTraffic *dstHost, HostAddr *dstAddr,
 			      TrafficCounter length, Counter numPkts,
 			      int actualDeviceId);
-
+extern u_int computeEfficiency(u_int pktLen);
+extern void updateGreEfficiency(HostTraffic *srcHost, HostTraffic *dstHost,
+				u_int numPkts, u_int numBytes, int actualDeviceId);
+extern void updateIpsecEfficiency(HostTraffic *srcHost, HostTraffic *dstHost,
+				  u_int numPkts, u_int numBytes, int actualDeviceId);
 extern void queuePacket(u_char * _deviceId, const struct pcap_pkthdr *h, const u_char *p);
 extern void cleanupPacketQueue(void);
 extern void *dequeuePacket(void* notUsed);
@@ -471,7 +483,7 @@ extern void initUserPrefs(UserPref *pref);
 
 /* util.c */
 extern void setEmptySerial(HostSerial *a);
-extern void handleAddressLists(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
+extern void handleAddressLists(char* addresses, NetworkStats theNetworks[MAX_NUM_NETWORKS],
 			       u_short *numNetworks, char *localAddresses,
 			       int localAddressesLen, int flagWhat);
 extern void handleFlowsSpecs(void);
@@ -563,6 +575,7 @@ extern unsigned short isLocalAddress(HostAddr *addr, u_int actualDeviceId,
 				     u_int32_t *the_local_network_mask);
 extern int dotted2bits(char *mask);
 extern void handleLocalAddresses(char* addresses);
+extern void handleKnownAddresses(char* addresses);
 extern unsigned short isPseudoLocalAddress(HostAddr *addr, u_int actualDeviceId,
 					   u_int32_t *the_local_network, 
 					   u_int32_t *the_local_network_mask);
@@ -570,13 +583,13 @@ extern unsigned short _pseudoLocalAddress(HostAddr *addr,
 					  u_int32_t *the_local_network, 
 					  u_int32_t *the_local_network_mask);
 extern unsigned short __pseudoLocalAddress(struct in_addr *addr,
-					   u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
+					   NetworkStats theNetworks[MAX_NUM_NETWORKS],
 					   u_short numNetworks,
 					   u_int32_t *the_local_network, 
 					   u_int32_t *the_local_network_mask);
 extern unsigned short deviceLocalAddress(HostAddr *addr, u_int deviceId,
-					   u_int32_t *the_local_network, 
-					   u_int32_t *the_local_network_mask);
+					 u_int32_t *the_local_network, 
+					 u_int32_t *the_local_network_mask);
 extern unsigned short isPseudoBroadcastAddress(HostAddr *addr,
 					       u_int32_t *the_local_network, 
 					       u_int32_t *the_local_network_mask);
@@ -614,6 +627,11 @@ extern int _lockHostsHashMutex(HostTraffic *host, char *where, char *file, int l
 #define unlockHostsHashMutex(a) _unlockHostsHashMutex(a, __FILE__, __LINE__)
 extern int _unlockHostsHashMutex(HostTraffic *host, char* file, int line);
 
+#define lockExclusiveHostsHashMutex(a, b) _lockExclusiveHostsHashMutex(a, b, __FILE__, __LINE__)
+extern int _lockExclusiveHostsHashMutex(HostTraffic *host, char *where, char *file, int line);
+#define unlockExclusiveHostsHashMutex(a) _unlockExclusiveHostsHashMutex(a, __FILE__, __LINE__)
+extern int _unlockExclusiveHostsHashMutex(HostTraffic *host, char* file, int line);
+
 #define setHolder(a) {							\
     if(fileName != NULL) { strncpy(a.file, fileName, sizeof(a.file)-1); a.file[sizeof(a.file)-1]='\0'; } else \
       { memset(&(a), 0, sizeof(a)); }					\
@@ -720,11 +738,6 @@ extern unsigned short computeIdx(HostAddr *srcAddr, HostAddr *dstAddr,
 				 int sport, int dport);
 extern u_int16_t computeTransId(HostAddr *srcAddr, HostAddr *dstAddr,
 				int sport, int dport);
-#ifdef MAKE_WITH_I18N
-char *i18n_xvert_locale2common(const char *input);
-char *i18n_xvert_acceptlanguage2common(const char *input);
-#endif /* MAKE_WITH_I18N */
-
 extern int setSpecifiedUser(void);
 extern u_short ip2AS(HostAddr ip);
 extern u_int16_t getHostAS(HostTraffic *el);
@@ -759,8 +772,8 @@ extern int snprintf(char *str, size_t n, const char *fmt, ...);
 		   (u_int32)*((u_int8_t *)x+0)<<0	\
 		   )
 #else
-#define ptohs(x) *(u_int16_t *)(x)
-#define ptohl(x) *(u_int32 *)(x)
+#define ptohs(x) (*(u_int16_t *)(x))
+#define ptohl(x) (*(u_int32 *)(x))
 #endif
 
 /* Conditional utility functions - code in util.c, activated if it's not already in some library */
@@ -778,12 +791,13 @@ extern char **buildargv(const char *argv);
 #ifndef HAVE_FREEARGV
 extern void freeargv(char **argv);
 #endif
-extern void handleWhiteBlackListAddresses(char* addresses, u_int32_t theNetworks[MAX_NUM_NETWORKS][4],
+extern void handleWhiteBlackListAddresses(char* addresses, 
+					  NetworkStats theNetworks[MAX_NUM_NETWORKS],
 					  u_short *numNets, char* outAddresses,
 					  int outAddressesLen);
 extern unsigned short isOKtoSave(u_int32_t addr,
-				 u_int32_t whiteNetworks[MAX_NUM_NETWORKS][4],
-				 u_int32_t blackNetworks[MAX_NUM_NETWORKS][4],
+				 NetworkStats whiteNetworks[MAX_NUM_NETWORKS],
+				 NetworkStats blackNetworks[MAX_NUM_NETWORKS],
 				 u_short numWhiteNets, u_short numBlackNets);
 extern float timeval_subtract(struct timeval x, struct timeval y);
 extern void freePortsUsage(HostTraffic *el);
@@ -810,7 +824,8 @@ extern void l7SessionProtoDetection(IPSession *theSession,
 /* sessions.c */
 #define checkSessionIdx(a) _checkSessionIdx(a, actualDeviceId, __FILE__, __LINE__)
 extern u_int _checkSessionIdx(u_int idx, int actualDeviceId, char* file, int line);
-extern void freeSession(IPSession *sessionToPurge, int actualDeviceId, u_char allocateMemoryIfNeeded, u_char lockMutex);
+extern void freeSession(IPSession *sessionToPurge, int actualDeviceId, 
+			u_char allocateMemoryIfNeeded, u_char lockMutex);
 extern void freeFcSession(FCSession *sessionToPurge, int actualDeviceId,
 			  u_char allocateMemoryIfNeeded, u_char lockMutex);
 extern void scanTimedoutTCPSessions(int actualDeviceId);
@@ -821,13 +836,21 @@ extern IPSession* handleSession(const struct pcap_pkthdr *h,
                                 u_short fragmentedData, u_int tcpWin,
                                 HostTraffic *srcHost, u_short sport,
                                 HostTraffic *dstHost, u_short dport,
-                                u_int length, struct tcphdr *tp,
+                                u_int sent_length, u_int rcvd_length /* Always 0 except for NetFlow v9 */,
+				struct tcphdr *tp,
                                 u_int tcpDataLength, u_char* packetData,
                                 int actualDeviceId, u_short *newSession,
 				u_char real_session /* vs. faked/netflow-session */);
 extern void updateHostUsers(char *userName, int userType, HostTraffic *theHost);
 extern void handlePluginSessionTermination(IPSession *sessionToPurge, int actualDeviceId);
-
+extern void updatePeersDelayStats(HostTraffic *peer_a, HostSerial *peer_b_serial,
+				  u_int16_t port,
+				  struct timeval *nwDelay,
+				  struct timeval *synAckTime, 
+				  struct timeval *ackTime,
+				  u_char is_client_delay, 
+				  int port_idx);
+extern void updateSessionDelayStats(IPSession* session);
 extern FCSession* handleFcSession(const struct pcap_pkthdr *h,
 				  u_short fragmentedData,
 				  HostTraffic *srcHost, HostTraffic *dstHost,
@@ -888,6 +911,10 @@ extern int h_errno; /* netdb.h */
 #define NTOHL(x)    (x) = ntohl(x)
 #endif
 
+#ifndef NTOHS
+#define NTOHS(x)    (x) = ntohs(x)
+#endif
+
 #ifndef BufferTooSmall
 #define BufferTooSmall(buf, len) traceEvent(CONST_TRACE_ERROR, "Buffer [buffer len=%d] too small @ %s:%d", len,  __FILE__,__LINE__)
 #endif
@@ -962,7 +989,7 @@ Code "inherited" from nslookup
 #define isValidVsanId(a)            ((a > 0) && (a < MAX_USER_VSAN))
 #define subnetLocalHost(a)          ((a != NULL) && FD_ISSET(FLAG_SUBNET_LOCALHOST, &(a->flags)))
 #define privateIPAddress(a)         ((a != NULL) && FD_ISSET(FLAG_PRIVATE_IP_ADDRESS, &(a->flags)))
-#define broadcastHost(a)            ((a != NULL) && (!isFcHost (a)) && ((cmpSerial(&a->hostSerial, &myGlobals.broadcastEntry->hostSerial) || FD_ISSET(FLAG_BROADCAST_HOST, &(a->flags))) || ((a->hostIp4Address.s_addr == 0) && (a->ethAddressString[0] == '\0'))))
+#define broadcastHost(a)            ((a != NULL) && (a != myGlobals.otherHostEntry) && (!isFcHost (a)) && ((cmpSerial(&a->hostSerial, &myGlobals.broadcastEntry->hostSerial) || FD_ISSET(FLAG_BROADCAST_HOST, &(a->flags))) || ((a->hostIp4Address.s_addr == 0) && (a->ethAddressString[0] == '\0'))))
 #define multicastHost(a)            ((a != NULL) && (!isFcHost (a)) && FD_ISSET(FLAG_MULTICAST_HOST, &(a->flags)))
 #define gatewayHost(a)              ((a != NULL) && FD_ISSET(FLAG_GATEWAY_HOST, &(a->flags)))
 #define nameServerHost(a)           ((a != NULL) && FD_ISSET(FLAG_NAME_SERVER_HOST, &(a->flags)))
