@@ -33,13 +33,7 @@
 #define TD_BG     ""
 #define TR_ON     "onMouseOver=\"this.bgColor = '#EDF3FE'\" onMouseOut =\"this.bgColor = '#FFFFFF'\""
 #define TABLE_DEFAULTS " CELLSPACING=0 CELLPADDING=2"
-#define DARK_BG   "BGCOLOR=\"#F3F3F3\"" /* If you change it change it into the
-                                         * stylesheet too */
-#define TOMATO_BG "BGCOLOR=\"#FF6347\""
-#define GOLD_BG   "BGCOLOR=\"#FFD700\""
-#define SALMON_BG "BGCOLOR=\"FA8072\""
-#define WHEAT_BG  "BGCOLOR=\"F5DEB3\""
-#define AZURE_BG  "BGCOLOR=\"F0FFFF\""
+#define DARK_BG   "BGCOLOR=\"#F3F3F3\"" /* If you change it change it into the stylesheet too */
 #endif
 
 
@@ -160,7 +154,8 @@ extern char* encodeString(char* in, char* out, u_int out_len);
 extern void printHTMLtrailer(void);
 extern void returnHTTPpageBadCommunity(void);
 extern void returnHTTPredirect(char* destination);
-void returnHTTPpageNotFound(char* additionalText);
+extern void returnHTTPpageNotFound(char* additionalText);
+extern void sendFile(char* fileName, int doNotUnlink);
 
 #define sendStringLen(a, b) _sendStringLen(a, b, 1)
 #define sendStringLenWOssi(a, b) _sendStringLen(a, b, 0)
@@ -205,7 +200,7 @@ extern void printMulticastStats(int sortedColumn /* ignored so far */,
                                 int revertOrder, int pageNum);
 extern void printVSANList(unsigned int deviceId);
 extern void printHostsInfo(int sortedColumn, int revertOrder, int pageNum, 
-			   int showBytes, int vlanId, int ifId);
+			   int showBytes, int vlanId, int ifId, int knownSubnetId);
 extern void makeDot(void);
 extern void printFcHostsInfo(int sortedColumn, int revertOrder, int pageNum, int showBytes, int vsanId);
 extern void printAllSessionsHTML (char* host, int actualDeviceId, int sortedColumn,
@@ -265,6 +260,10 @@ extern void handleNtopConfig (char *url, UserPrefDisplayPage page, int postLen);
 extern void purgeHost(char *serialized_serial);
 extern void printInterfaceStats(void);
 extern void findHost(char *key);
+extern char* hostRRdGraphLink(HostTraffic *el, 
+			      int network_mode,
+			      u_char is_subnet_host, 
+			      char *tmpStr, int tmpStrLen);
 
 /* webInterface.c */
 extern int execCGI(char* cgiName);
@@ -362,3 +361,4 @@ extern void drawVsanStatsPktsDistribution(int deviceId);
 extern void drawVsanDomainTrafficDistribution(u_short vsanId, u_char dataSent);
 extern void drawVsanSwilsProtoDistribution(u_short vsanId);
 
+extern int handlePerlHTTPRequest(char *url);
