@@ -70,15 +70,13 @@ u_int hashHost(HostAddr *hostIpAddress,  u_char *ether_addr,
   } else if(memcmp(ether_addr, myGlobals.broadcastEntry->ethAddress, LEN_ETHERNET_ADDRESS) == 0) {
     *el = myGlobals.broadcastEntry;
     return(BROADCAST_HOSTS_ENTRY);
-  } else if(hostIpAddress == NULL) {
+  } else if((hostIpAddress == NULL) 
+	    || isPseudoLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL)) {
     memcpy(&idx, &ether_addr[LEN_ETHERNET_ADDRESS-sizeof(u_int)], sizeof(u_int));
     (*useIPAddressForSearching) = 0;
   } else if(isBroadcastAddress(hostIpAddress, NULL, NULL)) {
     *el = myGlobals.broadcastEntry;
     return(BROADCAST_HOSTS_ENTRY);
-  } else if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL)) {
-    memcpy(&idx, &ether_addr[LEN_ETHERNET_ADDRESS-sizeof(u_int)], sizeof(u_int));
-    (*useIPAddressForSearching) = 0;
   } else {
     if(hostIpAddress != NULL) {
       if(myGlobals.runningPref.trackOnlyLocalHosts
@@ -157,13 +155,15 @@ static void freeHostSessions(HostTraffic *host, int theDevice) {
   if(host->l2Family == FLAG_HOST_TRAFFIC_AF_ETH) {
     for(i=0; i<MAX_TOT_NUM_SESSIONS; i++) {
       IPSession *prevSession, *nextSession, *theSession;
+      int mutex_idx;
 
       if(myGlobals.ntopRunState > FLAG_NTOPSTATE_RUN /* i.e. active, not cleanup */ )
 	return;
 
       if(host->numHostSessions == 0) return;
 
-      accessMutex(&myGlobals.tcpSessionsMutex, "freeHostSessions");
+      mutex_idx = i % NUM_SESSION_MUTEXES;
+      accessMutex(&myGlobals.tcpSessionsMutex[mutex_idx], "freeHostSessions");
 
       prevSession = theSession = myGlobals.device[theDevice].tcpSession[i];
 
@@ -192,7 +192,7 @@ static void freeHostSessions(HostTraffic *host, int theDevice) {
 	}
       } /* while */
 
-      releaseMutex(&myGlobals.tcpSessionsMutex);
+      releaseMutex(&myGlobals.tcpSessionsMutex[mutex_idx]);
       ntop_conditional_sched_yield(); /* Allow other threads to run */
     } /* for */
 
@@ -466,7 +466,8 @@ void freeHostInfo(HostTraffic *host, int actualDeviceId) {
 
   if(host->icmpInfo != NULL) free(host->icmpInfo);
   if(host->trafficDistribution != NULL) free(host->trafficDistribution);
-
+  if(host->clientDelay != NULL) free(host->clientDelay);
+  if(host->serverDelay != NULL) free(host->serverDelay);
   if(host->dnsDomainValue != NULL) free(host->dnsDomainValue);
   host->dnsDomainValue = NULL;
   if(host->dnsTLDValue != NULL) free(host->dnsTLDValue);
@@ -554,11 +555,10 @@ int is_host_ready_to_purge(int actDevice, HostTraffic *el, time_t now) {
   time_t withSessionPurgeTime = now-PARM_HOST_PURGE_MINIMUM_IDLE_ACTVSES;
   
   if(el->to_be_deleted
-     || (
-	 (el->refCount == 0)
-	 && ((!myGlobals.runningPref.rFileName) 
-	     &&  (((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
-		  || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime))))
+     || ((myGlobals.pcap_file_list == NULL)
+	 && (el->refCount == 0)
+	 && ((((el->numHostSessions == 0) && (el->lastSeen < noSessionPurgeTime))
+	      || ((el->numHostSessions > 0)  && (el->lastSeen < withSessionPurgeTime))))
 	 && (!broadcastHost(el)) && (el != myGlobals.otherHostEntry)
 	 && (myGlobals.device[actDevice].virtualDevice /* e.g. sFlow/NetFlow */
 	     || (!myGlobals.runningPref.stickyHosts)
@@ -778,17 +778,12 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
   u_int32_t the_local_network, the_local_network_mask;
 
   if((hostIpAddress == NULL) && (ether_addr == NULL)) {
-    traceEvent(CONST_TRACE_WARNING, "Both Ethernet and IP addresses are NULL in lookupHost()[%s:%d]", file, line);
+    traceEvent(CONST_TRACE_WARNING, 
+	       "Both Ethernet and IP addresses are NULL in lookupHost()[%s:%d]", 
+	       file, line);
     return(NULL);
   }
 
-#ifdef DEBUG
-  traceEvent(CONST_TRACE_NOISY, "DEBUG: lookupHost(%s, %s, m=%u, f=%u, dev=%d, vlan=%d)",
-             addrtostr(hostIpAddress),
-             etheraddr_string(ether_addr, buf),
-             checkForMultihoming, forceUsingIPaddress, actualDeviceId, vlanId);
-#endif
-
 #ifdef HASH_DEBUG
   hashSanityCheck();
 #endif
@@ -797,6 +792,21 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 		 &useIPAddressForSearching,
 		 &el, actualDeviceId);
 
+#ifdef DEBUG
+  if(0)
+    traceEvent(CONST_TRACE_NOISY, "DEBUG: lookupHost(%s, %s, m=%u, f=%u, dev=%d, vlan=%d) [idx=%d]",
+	       addrtostr(hostIpAddress),
+	       etheraddr_string(ether_addr, buf),
+	       checkForMultihoming, forceUsingIPaddress, actualDeviceId, vlanId, idx);
+  else
+    traceEvent(CONST_TRACE_WARNING, "DEBUG: lookupHost(%s, %s, isLocalAddress=%u, _pseudoLocalAddress=%u) [idx=%d]",
+	       hostIpAddress ? addrtostr(hostIpAddress) : "<null>",
+	       etheraddr_string(ether_addr, buf),
+	       hostIpAddress ? isLocalAddress(hostIpAddress, actualDeviceId, NULL, NULL) : 0,
+	       hostIpAddress ? _pseudoLocalAddress(hostIpAddress, NULL, NULL) : 0,
+	       idx);
+#endif
+
   /* Remember the side effect of above routine - if -o | --no-mac is set,\
    * useIPAddressForSearching is now 1
    */
@@ -837,8 +847,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
       if(useIPAddressForSearching == 0) {
 	/* compare with the ethernet-address then the IP address */
 	if(memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0) {
-	  if((hostIpAddress != NULL) &&
-	     (hostIpAddress->hostFamily == el->hostIpAddress.hostFamily)) {
+	  if((hostIpAddress != NULL) && (hostIpAddress->hostFamily == el->hostIpAddress.hostFamily)) {
 	    if((!isMultihomed) && checkForMultihoming) {
 	      /* This is a local address hence this is a potential multihomed host. */
 
@@ -852,7 +861,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	    }
 	    hostFound = 1;
 	    break;
-	  } else if(hostIpAddress == NULL){  /* Only Mac Addresses */
+	  } else if(hostIpAddress == NULL) {  /* Only Mac Addresses */
 	    hostFound = 1;
 	    break;
 	  } else { /* MAC match found and we have the IP - need to update... */
@@ -888,7 +897,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	}
       } else {
 	/* -o | --no-mac (or NetFlow, which doesn't have MACs) - compare with only the IP address */
-	if(addrcmp(&el->hostIpAddress, hostIpAddress) == 0) {
+	if((addrcmp(&el->hostIpAddress, hostIpAddress) == 0) 
+	   || (ether_addr && (memcmp(el->ethAddress, ether_addr, LEN_ETHERNET_ADDRESS) == 0))) {
 	  hostFound = 1;
 	  break;
 	}
@@ -900,29 +910,30 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 
   if(locked_mutex) releaseMutex(&myGlobals.hostsHashLockMutex);
 
-  if((hostFound == 1) && (vlanId != NO_VLAN) && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
+  if((hostFound == 1) && (vlanId != NO_VLAN) && (el->vlanId != NO_VLAN)
+     && (vlanId != el->vlanId) && (!isMultivlaned(el))) {
     FD_SET(FLAG_HOST_TYPE_MULTIVLANED, &el->flags);
+
     if(myGlobals.multipleVLANedHostCount == 0) {
-      traceEvent(CONST_TRACE_ERROR, "mVLAN: Host (identical IP/MAC) found on multiple VLANs");
+      traceEvent(CONST_TRACE_ERROR, "mVLAN: Host (identical IP/MAC) found on multiple VLANs [%d][%d]", vlanId, el->vlanId);
       traceEvent(CONST_TRACE_INFO,  "mVLAN: ntop continues but will consolidate and thus probably overcount this traffic");
       traceEvent(CONST_TRACE_NOISY, "mVLAN: Up to %d examples will be printed", MAX_MULTIPLE_VLAN_WARNINGS);
     }
-    if(++myGlobals.multipleVLANedHostCount < MAX_MULTIPLE_VLAN_WARNINGS)
-      traceEvent(CONST_TRACE_NOISY, "mVLAN: Device %d Host %s (%02x:%02x:%02x:%02x:%02x:%02x) VLANs %d and %d",
-                 actualDeviceId,
-                 addrtostr(hostIpAddress),
-                 ether_addr[0],
-                 ether_addr[1],
-                 ether_addr[2],
-                 ether_addr[3],
-                 ether_addr[4],
-                 ether_addr[5],
-                 min(vlanId, el->vlanId),
+
+    if(++myGlobals.multipleVLANedHostCount < MAX_MULTIPLE_VLAN_WARNINGS) {
+      if(ether_addr)
+	traceEvent(CONST_TRACE_NOISY, "mVLAN: Device %d Host %s (%02x:%02x:%02x:%02x:%02x:%02x) VLANs %d and %d",
+		   actualDeviceId,
+		   addrtostr(hostIpAddress),
+		   ether_addr[0], ether_addr[1], ether_addr[2],
+		   ether_addr[3], ether_addr[4], ether_addr[5],
+		   min(vlanId, el->vlanId),
                  max(vlanId, el->vlanId));
+    }
   }
 
   if(numRuns > myGlobals.device[actualDeviceId].hashListMaxLookups)
-    myGlobals.device[actualDeviceId].hashListMaxLookups = numRuns ;
+    myGlobals.device[actualDeviceId].hashListMaxLookups = numRuns;
 
   if(hostFound) {
     /* Existing host entry */
@@ -987,11 +998,10 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
       if(locked_mutex) unlockHostsHashMutex(myGlobals.device[actualDeviceId].hash_hostTraffic[idx]);
       return(NULL);
     }
-    memset(el->protoIPTrafficInfos, 0, len);
+    memset(el->protoIPTrafficInfos, 0, len);   
 
     el->magic = CONST_MAGIC_NUMBER;
     el->hostTrafficBucket = idx; /* Set the bucket index */
-    el->originalHostTrafficBucket = idx; /* Set the bucket index */
 
     /* traceEvent(CONST_TRACE_INFO, "new entry added at bucket %d", idx); */
 
@@ -1039,7 +1049,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 
 	  FD_CLR(FLAG_SUBNET_LOCALHOST, &el->flags);
 
-	  if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) FD_SET(FLAG_PRIVATE_IP_ADDRESS, &el->flags);
+	  if(isPrivateAddress(hostIpAddress, &the_local_network, &the_local_network_mask))
+	    FD_SET(FLAG_PRIVATE_IP_ADDRESS, &el->flags);
 
 	  if(!isBroadcastAddress(hostIpAddress, &the_local_network, &the_local_network_mask)) {
 	    if(isPseudoLocalAddress(hostIpAddress, actualDeviceId, &the_local_network, &the_local_network_mask))
@@ -1053,6 +1064,8 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	FD_CLR(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
       }
 
+      updateHostKnownSubnet(el);
+
       if(strncmp(el->ethAddressString, "FF:", 3) == 0) {
 	/*
 	  The trick below allows me not to duplicate the
@@ -1127,6 +1140,7 @@ HostTraffic* _lookupHost(HostAddr *hostIpAddress, u_char *ether_addr, u_int16_t
 	FD_SET(FLAG_SUBNET_PSEUDO_LOCALHOST, &el->flags);
 
       setResolvedName(el, el->hostNumIpAddress, FLAG_HOST_SYM_ADDR_TYPE_IP);
+      updateHostKnownSubnet(el);
 
       /* Trick to fill up the address cache */
       if(myGlobals.runningPref.numericFlag == 0)
@@ -1209,7 +1223,7 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
     return(NULL);
   }
 
-  idx = hashFcHost (hostFcAddress, vsanId, &el, actualDeviceId);
+  idx = hashFcHost(hostFcAddress, vsanId, &el, actualDeviceId);
 
   if(el != NULL) {
     return (el);
@@ -1240,8 +1254,8 @@ HostTraffic *lookupFcHost (FcAddress *hostFcAddress, u_short vsanId,
 		 idx, el->hostTrafficBucket);
     }
 
-    if ((el->fcCounters != NULL) &&
-	(memcmp ((u_int8_t *)&(el->fcCounters->hostFcAddress), hostFcAddress, LEN_FC_ADDRESS) == 0)) {
+    if((el->fcCounters != NULL) &&
+       (memcmp((u_int8_t *)&(el->fcCounters->hostFcAddress), hostFcAddress, LEN_FC_ADDRESS) == 0)) {
       hostFound = 1;
       break;
     }
@@ -1350,10 +1364,9 @@ static void dumpHash() {
 
   for(el=getFirstHost(myGlobals.actualReportDeviceId);
       el != NULL; el = getNextHost(myGlobals.actualReportDeviceId, el)) {
-    traceEvent(CONST_TRACE_INFO, "HASH_DEBUG: (%3d) %s / %s [bkt=%d][orig bkt=%d][next=0x%X]",
+    traceEvent(CONST_TRACE_INFO, "HASH_DEBUG: (%3d) %s / %s [bkt=%d][next=0x%X]",
 	       i++, el->ethAddressString, el->hostNumIpAddress,
-	       el->hostTrafficBucket, el->originalHostTrafficBucket,
-	       el->next);
+	       el->hostTrafficBucket, el->next);
   }
 }
 
@@ -1367,10 +1380,9 @@ static void hashSanityCheck() {
 
     while(el != NULL) {
       if(el->hostTrafficBucket != i)
-	traceEvent(CONST_TRACE_ERROR, "HASH: (%3d) %s / %s [bkt=%d][orig bkt=%d][next=0x%X]",
+	traceEvent(CONST_TRACE_ERROR, "HASH: (%3d) %s / %s [bkt=%d][next=0x%X]",
 		   i, el->ethAddressString, el->hostNumIpAddress,
-		   el->hostTrafficBucket, el->originalHostTrafficBucket,
-		   el->next);
+		   el->hostTrafficBucket, el->next);
       el = el->next;
     }
   }
@@ -1386,10 +1398,9 @@ static void hostHashSanityCheck(HostTraffic *host) {
 
     while(el != NULL) {
       if(el == host)
-	traceEvent(CONST_TRACE_ERROR, "HOST HASH: (%3d) %s / %s [bkt=%d][orig bkt=%d][next=0x%X]",
+	traceEvent(CONST_TRACE_ERROR, "HOST HASH: (%3d) %s / %s [bkt=%d][next=0x%X]",
 		   i, el->ethAddressString, el->hostNumIpAddress,
-		   el->hostTrafficBucket, el->originalHostTrafficBucket,
-		   el->next);
+		   el->hostTrafficBucket, el->next);
       el = el->next;
     }
   }
