@@ -149,7 +149,7 @@ void initIPServices(void) {
   memset(myGlobals.tcpSvc, 0, len);
 
   for(idx=0; myGlobals.configFileDirs[idx] != NULL; idx++) {
-    char tmpStr[64];
+    char tmpStr[256];
 
     safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "%s/services", myGlobals.configFileDirs[idx]);
     fd = fopen(tmpStr, "r");
@@ -310,7 +310,7 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].multicastPkts);
   resetTrafficCounter(&myGlobals.device[devIdx].ipPkts);
   resetTrafficCounter(&myGlobals.device[devIdx].ethernetBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].ipBytes);
+  resetTrafficCounter(&myGlobals.device[devIdx].ipv4Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].fragmentedIpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].tcpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].udpBytes);
@@ -319,12 +319,12 @@ void resetDevice(int devIdx, short fullReset) {
   resetTrafficCounter(&myGlobals.device[devIdx].dlcBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].ipxBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].stpBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].decnetBytes);
+  resetTrafficCounter(&myGlobals.device[devIdx].ipsecBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].netbiosBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].arpRarpBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].atalkBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].egpBytes);
-  resetTrafficCounter(&myGlobals.device[devIdx].osiBytes);
+  resetTrafficCounter(&myGlobals.device[devIdx].greBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].ipv6Bytes);
   resetTrafficCounter(&myGlobals.device[devIdx].otherBytes);
   resetTrafficCounter(&myGlobals.device[devIdx].fcBytes);
@@ -379,7 +379,7 @@ void resetDevice(int devIdx, short fullReset) {
   myGlobals.device[devIdx].last30daysThptIdx = 0;
   myGlobals.device[devIdx].hostsno = 0;
 
-  if(myGlobals.runningPref.rFileName == NULL) {
+  if(myGlobals.pcap_file_list == NULL) {
     myGlobals.device[devIdx].lastThptUpdate = myGlobals.device[devIdx].lastMinThptUpdate =
       myGlobals.device[devIdx].lastHourThptUpdate = myGlobals.device[devIdx].lastFiveMinsThptUpdate = time(NULL);
   }
@@ -428,16 +428,11 @@ void initCounters(void) {
   int numRead = 0;
   u_char compressedFormat;
   char buf[LEN_GENERAL_WORK_BUFFER];
-#ifdef MAKE_WITH_I18N
-  char *workLanguage;
-#ifdef HAVE_DIRENT_H
   struct dirent **dirList;
-  int j, iLang, nLang, found;
+  int iLang, nLang, found;
   DIR *testDirEnt;
   char *tmpStr;
   char* realLocale;
-#endif
-#endif
 
   setDomainName();
 
@@ -496,7 +491,7 @@ void initCounters(void) {
   myGlobals.numVendorLookupFoundMulticast = 0;
   myGlobals.numVendorLookupFoundLAA = 0;
 
-  if (myGlobals.runningPref.rFileName == NULL) {
+  if (myGlobals.pcap_file_list == NULL) {
       myGlobals.initialSniffTime = myGlobals.lastRefreshTime = time(NULL);
   }
   else {
@@ -581,173 +576,6 @@ void initCounters(void) {
       } else
           traceEvent(CONST_TRACE_NOISY, "ASN: ntop continues ok, but without ASN information.");
   }
-  /* i18n */
-#ifdef MAKE_WITH_I18N
-  /*
-   *  Obtain - from the os - the default locale.
-   */
-  workLanguage = setlocale(LC_ALL, "");
-  if(workLanguage != NULL ) {
-#ifdef I18N_DEBUG
-    traceEvent(CONST_TRACE_NOISY,
-	       "I18N: Default language (from ntop host) is '%s' (raw)\n",
-	       workLanguage);
-#endif
-    myGlobals.defaultLanguage = i18n_xvert_locale2common(workLanguage);
-    traceEvent(CONST_TRACE_INFO,
-	       "I18N: Default language (from ntop host) is '%s'\n",
-	       myGlobals.defaultLanguage);
-  } else {
-    traceEvent(CONST_TRACE_INFO,
-	       "I18N: Default language (from ntop host) is unspecified\n");
-    myGlobals.defaultLanguage = NULL;
-  }
-
-  /*
-   *  We initialize the array, myGlobals.supportedLanguages[] as follows...
-   *    We scan the directory entries in LOCALDIR, fix 'em up and then
-   *    see if the ntop html directory /html_cc[_XX]) exists?
-   *
-   *  Those that do, up to the limit, are loaded in the list...
-   *
-   *  We do not load an empty or artificial en - the logic in http.c will
-   *  present the default page (in .../html) if no language specific one is found.
-   *
-   *  We strip because we're not ready to support multiple char sets.  We're pretty
-   *  much just UTF-8
-   *
-   */
-
-#ifdef HAVE_DIRENT_H
-  nLang = scandir(locale_dir, &dirList, NULL, alphasort);
-  if(nLang < 0) {
-    traceEvent(CONST_TRACE_WARNING,
-	       "I18N: Error obtaining locale list, scandir(%s,...) errno is %d\n",
-	       locale_dir,
-	       errno);
-    traceEvent(CONST_TRACE_NOISY, "continues without multiple language support");
-  } else {
-    traceEvent(CONST_TRACE_NOISY, "I18N: scandir(%s,...) returned %d", locale_dir, nLang);
-    for (iLang=0; (iLang<nLang) && (myGlobals.maxSupportedLanguages < MAX_LANGUAGES_SUPPORTED); iLang++) {
-#ifdef I18N_DEBUG
-      traceEvent(CONST_TRACE_NOISY, "I18N_DEBUG: %2d. '%s'", iLang, dirList[iLang]->d_name);
-#endif
-      if(dirList[iLang]->d_name[0] == '.') {
-	/* skip parent/self directory entries */
-	continue;
-      }
-
-      if( (dirList[iLang]->d_type == DT_DIR) ||
-	  (dirList[iLang]->d_type == DT_LNK) ) {
-	tmpStr = i18n_xvert_locale2common(dirList[iLang]->d_name);
-
-	if(!strcmp(myGlobals.defaultLanguage, tmpStr)) {
-	  /* skip default language */
-	  traceEvent(CONST_TRACE_NOISY,
-		     "I18N_DEBUG: Skipping default language '%s' ('%s' raw)\n",
-		     tmpStr,
-		     dirList[iLang]->d_name);
-#ifdef I18N_DEBUG
-#endif
-	  free(tmpStr);
-	  continue;
-	}
-
-	found=0;
-	for (i=0; (!found) && i<myGlobals.maxSupportedLanguages; i++) {
-	  if(!strcmp(tmpStr, myGlobals.supportedLanguages[i])) {
-	    traceEvent(CONST_TRACE_NOISY,
-		       "I18N_DEBUG: Skipping already supported language, '%s'\n",
-		       dirList[iLang]->d_name);
-#ifdef I18N_DEBUG
-#endif
-	    found=1;
-	    break;
-	  }
-	}
-	if(!found) {
-	  traceEvent(CONST_TRACE_NOISY,
-		     "I18N: Testing locale '%s' (from '%s')\n",
-		     tmpStr,
-		     dirList[iLang]->d_name);
-	  for(i=0; (!found) && (myGlobals.dataFileDirs[i] != NULL); i++) {
-
-	    safe_snprintf(__FILE__, __LINE__, buf, sizeof(buf), "%s/html_%s",
-			myGlobals.dataFileDirs[i],
-			tmpStr);
-#ifdef WIN32
-	    j=0;
-	    while (buf[j] != '\0') {
-	      if(buf[j] == '/') buf[j] = CONST_PATH_SEP;
-	      j++;
-	    }
-#endif
-
-#ifdef I18N_DEBUG
-	    traceEvent(CONST_TRACE_NOISY,
-		       "I18N_DEBUG: Looking for directory '%s'\n",
-		       buf);
-#endif
-	    testDirEnt = opendir(buf);
-	    if(testDirEnt != NULL) {
-	      closedir(testDirEnt);
-
-	      realLocale = strdup(setlocale(LC_ALL, NULL));
-	      setlocale(LC_ALL, tmpStr);
-	      myGlobals.strftimeFormat[myGlobals.maxSupportedLanguages] = nl_langinfo(D_T_FMT);
-	      setlocale(LC_ALL, realLocale);
-	      free(realLocale);
-
-	      myGlobals.supportedLanguages[myGlobals.maxSupportedLanguages++] = strdup(tmpStr);
-	      found=1;
-	      traceEvent(CONST_TRACE_INFO,
-			 "I18N: '%s' ntop language files found, is supported.\n",
-			 tmpStr);
-	    }
-	  }
-
-	  if(!found) {
-	    traceEvent(CONST_TRACE_NOISY,
-		       "I18N: '%s' ntop language files not found, may not be supported.\n",
-		       tmpStr);
-	  }
-
-	  free(tmpStr);
-
-#ifdef I18N_DEBUG
-	} else {
-	  traceEvent(CONST_TRACE_NOISY,
-		     "I18N_DEBUG: Skipping duplicate locale '%s'\n",
-		     dirList[iLang]->d_name);
-#endif
-	}
-
-#ifdef I18N_DEBUG
-      } else {
-	traceEvent(CONST_TRACE_NOISY, "I18N_DEBUG: Skipping file '%s' (type %d)",
-		   dirList[iLang]->d_name,
-		   dirList[iLang]->d_type);
-#endif
-      }
-    }
-    for (iLang=0; iLang<nLang; iLang++) {
-      free(dirList[iLang]);
-    }
-    free(dirList);
-  }
-
-#else
-  traceEvent(CONST_TRACE_WARNING,
-             "I18N: Unable to scan locales (missing dirent.h at compile time) - ntop continues\n");
-#endif /* HAVE_DIRENT_H */
-
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY,
-             "I18N: This instance of ntop supports %d additional language(s)\n",
-             myGlobals.maxSupportedLanguages);
-#else
-  traceEvent(CONST_TRACE_ALWAYSDISPLAY,
-             "I18N: This instance of ntop does not support multiple languages\n");
-#endif /* MAKE_WITH_I18N */
 
   initIPCountryTable();
 }
@@ -768,18 +596,18 @@ void resetStats(int deviceId) {
   for(j=FIRST_HOSTS_ENTRY; j<myGlobals.device[deviceId].actualHashSize; j++) {
     HostTraffic *el = myGlobals.device[deviceId].hash_hostTraffic[j], *elNext;
 
-    if(el) lockHostsHashMutex(el, "resetStats");
+    if(el) lockExclusiveHostsHashMutex(el, "resetStats");
 
     while(el != NULL) {
       elNext = el->next;
 
       if((el != myGlobals.broadcastEntry) && (el != myGlobals.otherHostEntry)) {
-	unlockHostsHashMutex(el);
+	unlockExclusiveHostsHashMutex(el);
 	freeHostInfo(el, deviceId);
-	if(elNext) lockHostsHashMutex(elNext, "resetStats");
+	if(elNext) lockExclusiveHostsHashMutex(elNext, "resetStats");
       } else {
 	if(!elNext) 
-	  unlockHostsHashMutex(el);
+	  unlockExclusiveHostsHashMutex(el);
       }
 
       el = elNext;
@@ -801,10 +629,10 @@ void resetStats(int deviceId) {
   if(myGlobals.device[deviceId].fcSession != NULL) {
     for(j=0; j<MAX_TOT_NUM_SESSIONS; j++)
       if((session = myGlobals.device[deviceId].fcSession[j]) != NULL) {
-          for (i = 0; i < MAX_LUNS_SUPPORTED; i++) {
-              if (session->activeLuns[i] != NULL) {
-                  free (session->activeLuns[i]);
-              }
+          for(i = 0; i < MAX_LUNS_SUPPORTED; i++) {
+	    if(session->activeLuns[i] != NULL) {
+	      free (session->activeLuns[i]);
+	    }
           }
           free(session);
           myGlobals.device[deviceId].fcSession[j] = NULL;
@@ -829,8 +657,6 @@ void resetStats(int deviceId) {
     myGlobals.otherHostEntry->hostSerial.serialType = SERIAL_IPV4;
     myGlobals.otherHostEntry->hostSerial.value.ipSerial.ipAddress.Ip4Address.s_addr = -1;
     myGlobals.otherHostEntry->next = NULL;
-    FD_SET(FLAG_BROADCAST_HOST, &(myGlobals.broadcastEntry->flags));
-    myGlobals.otherHostEntry->next = NULL;
   }
 
   if(myGlobals.purgeMutex.isInitialized)
@@ -961,8 +787,10 @@ void reinitMutexes (void) {
  */
   createMutex(&myGlobals.logViewMutex);
   createMutex(&myGlobals.gdbmMutex);        /* data to synchronize thread access to db files */
-  createMutex(&myGlobals.tcpSessionsMutex); /* data to synchronize TCP sessions
-                                             * access */
+
+  for(i=0; i<NUM_SESSION_MUTEXES; i++)
+    createMutex(&myGlobals.tcpSessionsMutex[i]); /* data to synchronize TCP sessions access */
+
   createMutex(&myGlobals.fcSessionsMutex);
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
   createMutex(&myGlobals.purgePortsMutex);  /* data to synchronize port purge access */
@@ -991,12 +819,14 @@ void reinitMutexes (void) {
 void initThreads(void) {
   int i;
 
+  /*
   for(i=0; i<myGlobals.numDevices; i++) {
     createThread(&myGlobals.device[i].dequeuePacketThreadId, dequeuePacket, (void*)((long)i));
     traceEvent(CONST_TRACE_INFO, "THREADMGMT[t%lu]: NPA: Started thread for network packet analyzer (%s)",
 	       (long)myGlobals.device[i].dequeuePacketThreadId,
 	       myGlobals.device[i].humanFriendlyName);
   }
+  */
 
   /*
    * Create the thread (3) - SFP - Scan Fingerprints
@@ -1095,9 +925,8 @@ void allocDeviceMemory(int deviceId) {
  */
 void addDevice(char* deviceName, char* deviceDescr) {
   int i, deviceId, mallocLen, memlen;
-  NtopInterface *tmpDevice, *oldDevice;
   char *workDevices = NULL;
-  char myName[80], *column = NULL, ebuf[CONST_SIZE_PCAP_ERR_BUF], tmpStr[64];
+  char myName[255], *column = NULL, ebuf[CONST_SIZE_PCAP_ERR_BUF], tmpStr[64];
 
   ebuf[0] = '\0', myName[0] = '\0';
 
@@ -1107,7 +936,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
   }
 
   /* Remove unwanted characters */
- sanitizeIfName(deviceName);
+  sanitizeIfName(deviceName);
 
   traceEvent(CONST_TRACE_NOISY, "Adding network device %s", deviceName);
 
@@ -1115,13 +944,6 @@ void addDevice(char* deviceName, char* deviceDescr) {
     deviceId = createDummyInterface("none");
     traceEvent(CONST_TRACE_INFO, "-i none, so initialized only a dummy device");
   } else {
-    mallocLen = sizeof(NtopInterface)*(myGlobals.numDevices+1);
-    tmpDevice = (NtopInterface*)malloc(mallocLen);
-    memset(tmpDevice, 0, mallocLen);
-    memcpy(tmpDevice, myGlobals.device, sizeof(NtopInterface)*(myGlobals.numDevices));
-    oldDevice = myGlobals.device;
-    myGlobals.device = tmpDevice;
-    if (oldDevice != NULL) free(oldDevice);
     deviceId = myGlobals.numDevices;
 
     safe_snprintf(__FILE__, __LINE__, tmpStr, sizeof(tmpStr), "device.name.%s", deviceName);
@@ -1195,7 +1017,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
     }
 #endif
 
-    if (myGlobals.runningPref.printIpOnly) {
+    if(myGlobals.runningPref.printIpOnly) {
       myGlobals.device[deviceId].pcapPtr =
 	pcap_open_live(myGlobals.device[deviceId].name,
 		       myGlobals.runningPref.enablePacketDecoding == 0 ? 68 : DEFAULT_SNAPLEN,
@@ -1243,7 +1065,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
       }
 
     if(myGlobals.runningPref.enableSuspiciousPacketDump) {
-        if(myGlobals.runningPref.rFileName == NULL)
+        if(myGlobals.pcap_file_list == NULL)
 	  safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.dev%s.pcap",
 			myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
 			CONST_PATH_SEP,
@@ -1252,7 +1074,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
 			myGlobals.device[deviceId].name);
 	else
 	  safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.pcap",
-			myGlobals.runningPref.rFileName,
+			myGlobals.pcap_file_list,
 			CONST_PATH_SEP);
 
 	myGlobals.device[deviceId].pcapErrDumper = pcap_dump_open(myGlobals.device[deviceId].pcapPtr, myName);
@@ -1308,6 +1130,8 @@ void addDevice(char* deviceName, char* deviceDescr) {
       myGlobals.device[deviceId].netmask.s_addr = 0xFFFFFF00; /* dummy */
     }
 
+    addDeviceNetworkToKnownSubnetList(&myGlobals.device[deviceId]);
+
     if((myGlobals.device[deviceId].network.s_addr == 0) &&
        (myGlobals.device[deviceId].netmask.s_addr == 0xFFFFFFFF) ) {
       /* Unnumbered interface... */
@@ -1435,7 +1259,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
     struct in_addr myLocalHostAddress;
 
 
-    if((myGlobals.numDevices < MAX_NUM_DEVICES)
+    if((myGlobals.numDevices < (MAX_NUM_DEVICES-1))
        && strcmp(myGlobals.device[deviceId].name, "none")) {
       traceEvent(CONST_TRACE_INFO, "Checking %s for additional devices", myGlobals.device[deviceId].name);
       for(k=0; k<=MAX_NUM_DEVICES_VIRTUAL; k++) {
@@ -1446,14 +1270,6 @@ void addDevice(char* deviceName, char* deviceDescr) {
 	traceEvent(CONST_TRACE_NOISY, "Checking %s", tmpDeviceName);
 	if(getLocalHostAddress(&myLocalHostAddress, &netmask_v6, tmpDeviceName) == 0) {
 	  /* The virtual interface exists */
-
-	  mallocLen = sizeof(NtopInterface)*(myGlobals.numDevices+1);
-	  tmpDevice = (NtopInterface*)malloc(mallocLen);
-	  memset(tmpDevice, 0, mallocLen);
-	  memcpy(tmpDevice, myGlobals.device, sizeof(NtopInterface)*myGlobals.numDevices);
-	  free(myGlobals.device);
-	  myGlobals.device = tmpDevice;
-
 	  myGlobals.device[myGlobals.numDevices].ifAddr.s_addr = myLocalHostAddress.s_addr;
 	  if(myLocalHostAddress.s_addr == myGlobals.device[deviceId].ifAddr.s_addr)
 	    continue; /* No virtual Interfaces */
@@ -1478,8 +1294,7 @@ void addDevice(char* deviceName, char* deviceDescr) {
   initDeviceDatalink(deviceId);
 
   if((myGlobals.actualReportDeviceId == 0) && myGlobals.device[0].dummyDevice)
-    myGlobals.actualReportDeviceId = deviceId;
-  
+    myGlobals.actualReportDeviceId = deviceId;  
 }
 
 /* ******************************* */
@@ -1504,30 +1319,30 @@ int validInterface(char *name) {
  */
 void initDevices(char* devices) {
   char *tmpDev=NULL, *tmpDescr=NULL;
-  pcap_if_t *devpointer, *devpointer_original;
+  pcap_if_t *devpointer;
   char intNames[32][MAX_IF_NAME], intDescr[32][MAX_IF_NAME];
   int ifIdx = 0;
   int defaultIdx = -1;
   int found = 0, intfc;
   char ebuf[CONST_SIZE_PCAP_ERR_BUF];
-  char myName[80];
+  char myName[255];
 
   ebuf[0] = '\0', myName[0] = '\0';
 
-  traceEvent(CONST_TRACE_NOISY, "Initializing network devices");
+  traceEvent(CONST_TRACE_NOISY, "Initializing network devices [%s]", devices ? devices : "");
 
-  if(myGlobals.runningPref.rFileName != NULL) {
+  if(myGlobals.pcap_file_list != NULL) {
     createDummyInterface("none");
     myGlobals.device[0].dummyDevice = 0;
-    myGlobals.device[0].pcapPtr  = pcap_open_offline(myGlobals.runningPref.rFileName, ebuf);
+    myGlobals.device[0].pcapPtr  = pcap_open_offline(myGlobals.pcap_file_list->fileName, ebuf);
 
     if(myGlobals.device[0].pcapPtr == NULL) {
       traceEvent(CONST_TRACE_ERROR, "pcap_open_offline(%s): '%s'",
-		 myGlobals.runningPref.rFileName, ebuf);
+		 myGlobals.pcap_file_list->fileName, ebuf);
       exit(-1);
     } else {
       if(myGlobals.device[0].humanFriendlyName != NULL) free(myGlobals.device[0].humanFriendlyName);
-      myGlobals.device[0].humanFriendlyName = strdup(myGlobals.runningPref.rFileName);
+      myGlobals.device[0].humanFriendlyName = strdup(myGlobals.pcap_file_list->fileName);
       calculateUniqueInterfaceName(0);
     }
 
@@ -1535,7 +1350,7 @@ void initDevices(char* devices) {
     initDeviceDatalink(0);
 
     if(myGlobals.runningPref.enableSuspiciousPacketDump) {
-      if(myGlobals.runningPref.rFileName == NULL)
+      if(myGlobals.pcap_file_list == NULL)
 	safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.%s.pcap",
 		      myGlobals.runningPref.pcapLogBasePath, /* Added by Ola Lundqvist <opal@debian.org> */
 		      CONST_PATH_SEP,
@@ -1543,8 +1358,9 @@ void initDevices(char* devices) {
 		      myGlobals.device[0].uniqueIfName :
 		      myGlobals.device[0].name);
       else
-	safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s.ntop-suspicious-pkts.pcap",
-		      myGlobals.runningPref.rFileName);
+	safe_snprintf(__FILE__, __LINE__, myName, sizeof(myName), "%s%cntop-suspicious-pkts.pcap",
+		      myGlobals.runningPref.pcapLogBasePath,    /* Added by David Moore */
+		      CONST_PATH_SEP);
 
         myGlobals.device[0].pcapErrDumper = pcap_dump_open(myGlobals.device[0].pcapPtr, myName);
 
@@ -1562,12 +1378,12 @@ void initDevices(char* devices) {
 
   if(pcap_findalldevs(&devpointer, ebuf) < 0) {
     traceEvent(CONST_TRACE_ERROR, "pcap_findalldevs() call failed [%s]", ebuf);
-    traceEvent(CONST_TRACE_ERROR, "Have you instaled libpcap/winpcap properly?");
+    traceEvent(CONST_TRACE_ERROR, "Have you installed libpcap/winpcap properly?");
     return;
   } else {
     int i;
 
-    devpointer_original = devpointer;
+    myGlobals.allDevs = devpointer; /* save listhead for later use */
     for(i = 0; devpointer != 0; i++) {
       traceEvent(CONST_TRACE_NOISY, "Found interface [index=%d] '%s'", ifIdx, devpointer->name);
 
@@ -1613,11 +1429,9 @@ void initDevices(char* devices) {
     }
   }
 
-  pcap_freealldevs(devpointer_original);
-
   if(devices != NULL) {
     /* User has specified devices in the parameter list */
-    char *workDevices = strdup(devices), *strtokState;
+    char *workDevices = strdup(devices), *strtokState = NULL;
     int warnedVirtual = 0;
 
     tmpDev = strtok_r(workDevices, ",", &strtokState);
@@ -1878,29 +1692,18 @@ void startSniffer(void) {
 /* ***************************** */
 
 u_int createDummyInterface(char *ifName) {
-  u_int mallocLen, deviceId = myGlobals.numDevices;
-  NtopInterface *tmpDevice;
+#ifdef NOT_YET
+  u_int mallocLen;
+#endif
+  u_int deviceId = myGlobals.numDevices;
 
   traceEvent(CONST_TRACE_INFO, "Creating dummy interface, '%s'", ifName);
 
-  mallocLen = sizeof(NtopInterface)*(myGlobals.numDevices+1);
-  tmpDevice = (NtopInterface*)malloc(mallocLen);
-
-  if(tmpDevice == NULL)
-    return(-1);
-
-  memset(tmpDevice, 0, mallocLen);
-
-  if(myGlobals.numDevices > 0) {
-    NtopInterface *old_ptr = myGlobals.device;
-    memcpy(tmpDevice, myGlobals.device,
-	   sizeof(NtopInterface)*myGlobals.numDevices);
-    myGlobals.device = tmpDevice;
-    free(old_ptr);
+  if(myGlobals.numDevices >= (MAX_NUM_DEVICES-1)) {
+    traceEvent(CONST_TRACE_WARNING, "Too many devices: device '%s' can't be created", ifName);
   } else
-    myGlobals.device = tmpDevice;
+    myGlobals.numDevices++;
 
-  myGlobals.numDevices++;
   memset(&myGlobals.device[deviceId], 0, sizeof(NtopInterface));
 
   resetDevice(deviceId, 1);
