@@ -23,9 +23,7 @@
 #ifdef MAKE_STATIC_PLUGIN
 extern PluginInfo* icmpPluginEntryFctn(void);
 extern PluginInfo* sflowPluginEntryFctn(void);
-#ifndef EMBEDDED
 extern PluginInfo* rrdPluginEntryFctn(void);
-#endif
 extern PluginInfo* netflowPluginEntryFctn(void);
 #endif
 
@@ -47,8 +45,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
   PluginInfo* (*pluginJumpFunc)();
 #endif
   FlowFilterList *newFlow,
-                 *work,
-                 *prev;
+    *work, *prev = NULL;
 
   safe_snprintf(__FILE__, __LINE__, pluginPath, sizeof(pluginPath), "%s/%s", dirName != NULL ? dirName : ".", pluginName);
 
@@ -63,7 +60,7 @@ static void loadPlugin(char* dirName, char* pluginName) {
     return;
   }
 
-  pluginEntryFctnPtr = (void*)dlsym(pluginPtr, "PluginEntryFctn" /* CONST_PLUGIN_ENTRY_FCTN_NAME */);
+  pluginEntryFctnPtr = (void*)dlsym(pluginPtr, CONST_PLUGIN_ENTRY_FCTN_NAME);
 
   if(pluginEntryFctnPtr == NULL) {
 #ifdef WIN32
@@ -86,10 +83,8 @@ static void loadPlugin(char* dirName, char* pluginName) {
     pluginInfo = sflowPluginEntryFctn();
   else if(strcmp(pluginName, "netflowPlugin") == 0)
     pluginInfo = netflowPluginEntryFctn();
-#ifndef EMBEDDED
   else if(strcmp(pluginName, "rrdPlugin") == 0)
     pluginInfo = rrdPluginEntryFctn();
-#endif
   else
     pluginInfo = NULL;
 
@@ -244,9 +239,7 @@ void loadPlugins(void) {
   loadPlugin(NULL, "icmpPlugin");
   loadPlugin(NULL, "sflowPlugin");
   loadPlugin(NULL, "netflowPlugin");
-#ifndef EMBEDDED
   loadPlugin(NULL, "rrdPlugin");
-#endif
 #endif /* MAKE_STATIC_PLUGIN */
 }
 
