@@ -317,15 +317,15 @@ void updateDeviceThpt(int deviceToUpdate, int quickUpdate) {
 
     /* ******************************** */
 
-    el->actualRcvdPktThpt    = (float)(el->pktRcvd.value-el->lastPktRcvd.value)/timeDiff;
+    el->actualRcvdPktThpt    = (float)(el->pktRcvd.value-el->lastThptPktRcvd.value)/timeDiff;
     if(el->peakRcvdPktThpt   < el->actualRcvdPktThpt) el->peakRcvdPktThpt = el->actualRcvdPktThpt;
-    el->actualSentPktThpt    = (float)(el->pktSent.value-el->lastPktSent.value)/timeDiff;
+    el->actualSentPktThpt    = (float)(el->pktSent.value-el->lastThptPktSent.value)/timeDiff;
     if(el->peakSentPktThpt   < el->actualSentPktThpt) el->peakSentPktThpt = el->actualSentPktThpt;
-    el->actualTPktThpt       = (float)(el->pktRcvd.value-el->lastPktRcvd.value+
-				       el->pktSent.value-el->lastPktSent.value)/timeDiff;
+    el->actualTPktThpt       = (float)(el->pktRcvd.value-el->lastThptPktRcvd.value+
+				       el->pktSent.value-el->lastThptPktSent.value)/timeDiff;
     if(el->peakTPktThpt      < el->actualTPktThpt) el->peakTPktThpt = el->actualTPktThpt;
-    el->lastPktSent          = el->pktSent;
-    el->lastPktRcvd          = el->pktRcvd;
+    el->lastThptPktSent          = el->pktSent;
+    el->lastThptPktRcvd          = el->pktRcvd;
 
     /* ******************************** */
 
@@ -548,7 +548,7 @@ int isMatrixHost(HostTraffic *host, int actualDeviceId) {
 /* ******************************* */
 
 unsigned int matrixHostHash(HostTraffic *host, int actualDeviceId, int rehash) {
-  unsigned long hash;
+  unsigned long hash = 0;
   char tmpBuf[80], *str;
   int c;
 
@@ -737,8 +737,8 @@ char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len) {
 
   key = gdbm_firstkey(myGlobals.prefsFile);
   while (key.dptr) {
-    char val[256], localAddresses[1024], *communityName;
-    u_int32_t localNetworks[MAX_NUM_NETWORKS][4]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
+    char val[256], localAddresses[2048], *communityName;
+    NetworkStats localNetworks[MAX_NUM_NETWORKS]; /* [0]=network, [1]=mask, [2]=broadcast, [3]=mask_v6 */
     u_short numLocalNetworks = 0, i;
     
     if((fetchPrefsValue(key.dptr, val, sizeof(val)) == 0)
@@ -752,7 +752,7 @@ char* findHostCommunity(u_int32_t host_ip, char *buf, u_short buf_len) {
 
       // traceEvent(CONST_TRACE_WARNING, "--> Community %s has %d entries", communityName, numLocalNetworks);
       for(i=0; i<numLocalNetworks; i++) {
-	if((host_ip & localNetworks[i][1]) == localNetworks[i][0]) {
+	if((host_ip & localNetworks[i].address[1]) == localNetworks[i].address[0]) {
 	  //traceEvent(CONST_TRACE_WARNING, "--> Found community %s [%d]", communityName, numLocalNetworks);
 	  snprintf(buf, buf_len, "%s", communityName);
 	  return(buf);
